function [accuracy, finished] = Classifier(featurePath, resultPath, resultFileName, ...
    resultFileExpansion, times, trainingSamplesPerCategory, C)

[feature, going] = LoadFeatures(featurePath, false);
if (going)
    if (~isdir(resultPath))
        mkdir(resultPath);
    end
    categories = feature.categories;
    totalSamples = feature.images;
    trainingSamples = trainingSamplesPerCategory * categories;
    testingSamples = totalSamples - trainingSamples;
    testingSamplesPerCategory = zeros(1, categories);
    maxTestingSamplesPerCategory = 0;
    for i = 1: 1: categories
        testingSamplesPerCategory(i) = feature.cate(i).images - trainingSamplesPerCategory;
        if (maxTestingSamplesPerCategory < testingSamplesPerCategory(i))
            maxTestingSamplesPerCategory = testingSamplesPerCategory(i);
        end
    end
    len = length(C);
    accuracyPerCategory = zeros(len, times, categories);
    cateAcc = zeros(len, times);
    imageAcc = zeros(len, times);
    accuracy = zeros(1, len);
    for t = 1: 1: times
        fprintf('Testing process No.%d begins.\n', t);
        trainingSample = zeros(categories, trainingSamplesPerCategory);
        testingSample = zeros(categories, maxTestingSamplesPerCategory);
        for i = 1: 1: categories
            perm = randperm(feature.cate(i).images);
            trainingSample(i, :) = perm(1: trainingSamplesPerCategory);
            testingSample(i, 1: feature.cate(i).images - trainingSamplesPerCategory) = ...
                perm(trainingSamplesPerCategory + 1: feature.cate(i).images);
        end
        fprintf('Testing environment is successfully constructed.\n');
        fprintf('%d training samples (%d per category) and %d testing samples in total.\n', ...
            trainingSamples, trainingSamplesPerCategory, testingSamples);
        dimensions = feature.dimensions;
        trainingFeature = zeros(trainingSamples, dimensions);
        trainingLabel = zeros(1, trainingSamples);
        trainingCount = 0;
        testingFeature = zeros(testingSamples, dimensions);
        testingLabel = zeros(1, testingSamples);
        testingCount = 0;
        for i = 1: 1: categories
            for j = 1: 1: trainingSamplesPerCategory
                trainingCount = trainingCount + 1;
                vector = load(feature.cate(i).image(trainingSample(i, j)).path);
                trainingFeature(trainingCount, :) = vector.features;
                trainingLabel(trainingCount) = i;
            end
            for j = 1: 1: testingSamplesPerCategory(i)
                testingCount = testingCount + 1;
                vector = load(feature.cate(i).image(testingSample(i, j)).path);
                testingFeature(testingCount, :) = vector.features;
                testingLabel(testingCount) = i;
            end
        end
        for s = 1: 1: len
            options = ['-c ' num2str(C(s)) ', -q'];
            model = train(double(trainingLabel)', sparse(trainingFeature), options);
            testingResult = predict(testingLabel', sparse(testingFeature), model);
            fprintf('Classification based on SVM is finished. Statistics will be done.\n');
            testingCount = 0;
            confusionMatrix = zeros(categories, categories);
            correctSamples = 0;
            for i = 1: 1: categories
                for j = 1: 1: testingSamplesPerCategory(i)
                    testingCount = testingCount + 1;
                    confusionMatrix(i, testingResult(testingCount)) = ...
                        confusionMatrix(i, testingResult(testingCount)) + 1;
                end
                accuracyPerCategory(s, t, i) = confusionMatrix(i, i) / testingSamplesPerCategory(i);
                correctSamples = correctSamples + confusionMatrix(i, i);
                fprintf('    Category #%03d: The accuracy is %f%%. (Name is %s)\n', ...
                    i, accuracyPerCategory(s, t, i) * 100, feature.cate(i).name);
            end
            cateAcc(s, t) = mean(accuracyPerCategory(s, t, :));
            imageAcc(s, t) = correctSamples / testingSamples;
            fprintf('The mean accuracy by categories is %f%%.\n', cateAcc(s, t) * 100);
            fprintf('The mean accuracy by images is %f%%.\n', imageAcc(s, t) * 100);
            fprintf('Testing process No.%d is finished.\n', t);
        end
    end
    for s = 1: 1: len
        fprintf('All the testing is finished.\n');
        fprintf('The mean accuracy of all rounds by categories is %f%%.\n', ...
            mean(cateAcc(s, :)) * 100);
        fprintf('The std deviation of all rounds by categories is %f%%.\n', ...
            std(cateAcc(s, :)) * 100);
        fprintf('The mean accuracy of all rounds by images is %f%%.\n', ...
            mean(imageAcc(s, :)) * 100);
        fprintf('The std deviation of all rounds by images is %f%%.\n', ...
            std(imageAcc(s, :)) * 100);
        fprintf('All the testing is finished.\n');
        accuracy(s) = mean(cateAcc(s, :));
    end
    for s = 1: 1: len
        resultFile = [resultFileName num2str(C(s)) resultFileExpansion];
        file = fullfile(resultPath, resultFile);
        output = fopen(file, 'w');
        for t = 1: 1: times
            fprintf(output, 'Testing process No.%d begins.\r\n', t);
            for i = 1: 1: categories
                fprintf(output, '    Category #%03d: The accuracy is %f%%. (Name is %s)\r\n', ...
                    i, accuracyPerCategory(s, t, i) * 100, feature.cate(i).name);
            end
            fprintf(output, 'The mean accuracy by categories is %f%%.\r\n', cateAcc(s, t) * 100);
            fprintf(output, 'The mean accuracy by images is %f%%.\r\n', imageAcc(s, t) * 100);
            fprintf(output, 'Testing process No.%d is finished.\r\n', t);
            fprintf(output, '\r\n');
        end
        fprintf(output, 'All the testing is finished.\r\n');
        fprintf(output, 'The mean accuracy of all rounds by categories is %f%%.\r\n', ...
            mean(cateAcc(s, :)) * 100);
        fprintf(output, 'The std deviation of all rounds by categories is %f%%.\r\n', ...
            std(cateAcc(s, :)) * 100);
        fprintf(output, 'The mean accuracy of all rounds by images is %f%%.\r\n', ...
            mean(imageAcc(s, :)) * 100);
        fprintf(output, 'The std deviation of all rounds by images is %f%%.\r\n', ...
            std(imageAcc(s, :)) * 100);
        fprintf(output, 'All the testing is finished.\r\n');
        fclose(output);
    end
    finished = true;
else
    finished = false;
end

end
