function finished = DenseSIFT(imagePath, descriptorPath, binSize, mag, maxSize, colorOption)

scale = binSize / mag;

descriptor = [];
descriptor.categories = 0;
descriptor.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract SIFT descriptors.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            descriptor.categories = descriptor.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', descriptor.categories, categoryName);
            ca = descriptor.categories;
            descriptor.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            descriptorDirectory = fullfile(descriptorPath, categoryName);
            if (~isdir(descriptorDirectory))
                mkdir(descriptorDirectory);
            end
            for j = 1: 1: length(imageFiles)
                descriptor.cate(ca).images = descriptor.cate(ca).images + 1;
                imageFile = fullfile(imagePath, categoryName, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                descriptorFile = fullfile(descriptorDirectory, [name_, '.mat']);
                if (~exist(descriptorFile, 'file'))
                    image = imread(imageFile);
                    scale1 = maxSize / size(image, 1);
                    scale2 = maxSize / size(image, 2);
                    if (scale1 < scale2)
                        image = imresize(image, scale1);
                    else
                        image = imresize(image, scale2);
                    end
                    if (size(image, 3) == 3)
                        rgbimage = rgb2gray(image);
                    else
                        rgbimage = image;
                    end
                    image = single(image);
                    [SIFT.height, SIFT.width, parts] = size(image);
                    if (parts == 1)
                        image = repmat(image, [1, 1, 3]);
                    end
                    O = zeros(size(image));
                    O(:, :, 1) = (image(:, :, 1) - image(:, :, 2)) / (2 ^ 0.5);
                    O(:, :, 2) = (image(:, :, 1) + image(:, :, 2) - image(:, :, 3) * 2) / (6 ^ 0.5);
                    O(:, :, 3) = (image(:, :, 1) + image(:, :, 2) + image(:, :, 3)) / (3 ^ 0.5);
                    if (strcmp(colorOption, 'Gray'))
                        data = rgbimage;
                        SIFT.parts = 1;
                    elseif (strcmp(colorOption, 'RGB'))
                        data = image;
                        SIFT.parts = 3;
                    elseif (strcmp(colorOption, 'C'))
                        [x, y] = find(O(:, :, 3) == 0);
                        for px = 1: 1: length(x)
                            O(x(px), y(px), 3) = 1;
                        end
                        data = zeros(SIFT.height, SIFT.width, 2);
                        data(:, :, 1) = O(:, :, 1) ./ O(:, :, 3);
                        data(:, :, 2) = O(:, :, 2) ./ O(:, :, 3);
                        SIFT.parts = 2;
                    elseif (strcmp(colorOption, 'Opp'))
                        data = O;
                        SIFT.parts = 3;
                    end
                    data = single(data);
                    SIFT.l = [];
                    SIFT.d = [];
                    for p = 1: 1: SIFT.parts
                        [part.l, part.d] = vl_dsift(vl_imsmooth( ...
                            data(:, :, p), sqrt(max(scale .^ 2 - 0.25, 0))), 'size', binSize, 'step', mag, 'floatdescriptors');
                        part.l = part.l';
                        part.d = part.d';
                        for l = 1: 1: size(part.d, 1)
                            ss = sum(part.d(l, :));
                            if (ss > 0)
                                part.d(l, :) = part.d(l, :) / ss;
                            end
                            part.d(l, :) = part.d(l, :) .^ (1 / 2);
                        end
                        part.d = part.d * 512;
                        if (p == 1)
                            SIFT.l = part.l;
                        end
                        SIFT.d = cat(2, SIFT.d, part.d);
                    end
                    save(descriptorFile, 'SIFT');
                    fprintf('    Image #%03d out of %d images of category %03d is finished, %05d patches have been extracted.\n', ...
                        j, length(imageFiles), ca, size(SIFT.l, 1));
                end
            end
            descriptor.images = descriptor.images + descriptor.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                descriptor.categories, categoryName, descriptor.images);
        end
    end
    fprintf('Extraction of SIFT descriptors is finished.\n');
    finished = true;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
