function finished = KMeans(descriptorPath, codebookPath, codebookFilename, ...
    KMeans, sepCluster, maxImagesPerCategory, init, maxIterations)

[descriptor, going] = LoadDescriptors(descriptorPath, maxImagesPerCategory, false);

if (going)
    codebookDirectory = codebookPath;
    if (~isdir(codebookDirectory))
        mkdir(codebookDirectory);
    end
    codebookFile = fullfile(codebookPath, codebookFilename);
    if (~exist(codebookFile, 'file'))
        fprintf('Begin collecting points for clustering.\n');
        categories = descriptor.categories;
        if (~sepCluster)
            point = [];
            points = 0;
            for i = 1: 1: categories
                cate = descriptor.cate(i);
                images = cate.images;
                count = images;
                if (maxImagesPerCategory >= 0 && maxImagesPerCategory < count)
                    count = maxImagesPerCategory;
                end
                perm = randperm(images);
                for j = 1: 1: count
                    image = cate.image(perm(j));
                    SIFTDescriptor = load(image.path);
                    SIFT = SIFTDescriptor.SIFT;
                    points = points + size(SIFT.l, 1);
                    point = cat(1, point, SIFT.d);
                end
            end
            fprintf('Clustering begins, there are %d points in total.\n', points);
            tic;
            [center, ~, ~] = vl_kmeans(point', KMeans, 'Initialization', init, 'MaxNumIterations', maxIterations);
            toc;
            center = center'; %#ok<NASGU>
        else
            center = [];
            for i = 1: 1: categories
                point = [];
                points = 0;
                cate = descriptor.cate(i);
                images = cate.images;
                count = images;
                if (maxImagesPerCategory < count)
                    count = maxImagesPerCategory;
                end
                perm = randperm(images);
                for j = 1: 1: count
                    image = cate.image(perm(j));
                    SIFTDescriptor = load(image.SIFT);
                    SIFT = SIFTDescriptor.SIFT;
                    points = points + size(SIFT.l, 1);
                    point = cat(1, point, SIFT.d);
                end
                fprintf('Clustering for Category No.%d begins, there are %d points in total.\n', i, points);
                [sCenter, ~, ~] = vl_kmeans(point', KMeans, 'Initialization', init, 'MaxNumIterations', maxIterations);
                center = cat(1, center, sCenter');
            end
        end
        save(codebookFile, 'center');
        entries = KMeans;
        if (sepCluster)
            entries = entries * categories;
        end
        fprintf('Codebook with %d entries has been constructed.\n', entries);
    else
        fprintf('Codebook is existed.\n');
    end
    finished = true;
else
    finished = false;
end

end
