function finished = PoolingPatch(descriptorPath, codebookPath, codebookFilename, orientationPath, ...
    featurePath, KNN, beta, LP, NP, normalized, pyramid, aBins, pBins)

[~, going] = LoadDescriptors(descriptorPath, 0, false);
if (going)
    codebookFile = fullfile(codebookPath, codebookFilename);
    if (exist(codebookFile, 'file'))
        center = load(codebookFile);
        center = center.center;
        centers = size(center, 1);
        fprintf('Codebook is succesfully constructed. %d entries in total.\n', centers);
    else
        fprintf('Bad codebook file. Please check if %s exists.\n', codebookFile);
        going = false;
    end
end

if (going)
    if (exist(descriptorPath, 'dir'))
        fprintf('The pooling step begins.\n');
        categoryDirectories = dir(descriptorPath);
        totalImages = 0;
        categories = 0;
        for i = 1: 1: length(categoryDirectories)
            categoryName = categoryDirectories(i).name;
            if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
                categories = categories + 1;
                fprintf('Category # %03d: %s, begins.\n', categories, categoryName);
                for aa = aBins
                    for pp = pBins
                        featureDirectory = fullfile(featurePath, [num2str(KNN), ',', ...
                            num2str(length(pyramid)), 'L', ',', '(', num2str(aa), '_', num2str(pp), ')O'], categoryName);
                        if (~isdir(featureDirectory))
                            mkdir(featureDirectory);
                        end
                    end
                end
                imageDirectory = fullfile(descriptorPath, categoryName);
                imageFiles = dir(fullfile(imageDirectory, '*.mat'));
                images = length(imageFiles);
                orientationDirectory = fullfile(orientationPath, categoryName);
                for j = 1: 1: images
                    path = fullfile(descriptorPath, categoryName, imageFiles(j).name);
                    [~, name_] = fileparts(imageFiles(j).name);
                    featureFile = fullfile(featureDirectory, [name_, '.mat']);
                    orientationFile = fullfile(orientationDirectory, [name_, '.mat']);
                    if (~exist(featureFile, 'file'))
                        SIFTDescriptor = load(path);
                        SIFT = SIFTDescriptor.SIFT;
                        points = size(SIFT.d, 1);
                        desc2 = sum(SIFT.d .* SIFT.d, 2);
                        middle = SIFT.d * center';
                        center2 = sum(center .* center, 2);
                        dist = repmat(desc2, 1, centers) - 2 * middle + repmat(center2', points, 1);
                        index = zeros(points, KNN);
                        for l = 1: 1: points
                            d = dist(l, :);
                            [~, ind] = sort(d, 'ascend');
                            index(l, :) = ind(1: KNN);
                        end
                        unit = eye(KNN, KNN);
                        coef = zeros(points, centers);
                        for l = 1: 1: points
                            ind = index(l, :);
                            z = center(ind, :) - repmat(SIFT.d(l, :), KNN, 1);
                            C = z * z';
                            C = C + unit * beta * trace(C);
                            w = C \ ones(KNN, 1);
                            w = w / sum(w);
                            coef(l, ind) = w';
                        end
                        if (LP ~= -1)
                            coef = coef .^ LP;
                        end
                        times = sum(pyramid .^ 2);
                        features1 = zeros(times, centers);
                        count = 0;
                        for t = 1: 1: length(pyramid)
                            width = SIFT.width / pyramid(t);
                            height = SIFT.height / pyramid(t);
                            x = ceil(SIFT.l(:, 1) / width);
                            y = ceil(SIFT.l(:, 2) / height);
                            bin = (y - 1) * pyramid(t) + x;
                            for s = 1: 1: pyramid(t) ^ 2
                                count = count + 1;
                                ind = find(bin == s);
                                if (~isempty(ind))
                                    if (LP == -1)
                                        features1(count, :) = max(coef(ind, :), [], 1);
                                    else
                                        features1(count, :) = (sum(coef(ind, :), 1) / length(ind)) .^ (1 / LP);
                                    end
                                end
                            end
                        end
                        features1 = features1(:)';
                        orientation = load(orientationFile);
                        orientation = orientation.orientation;
                        for aa = aBins
                            for pp = pBins
                                featureDirectory = fullfile(featurePath, [num2str(KNN), ',', ...
                                    num2str(length(pyramid)), 'L', ',', '(', num2str(aa), '_', num2str(pp), ')O'], categoryName);
                                featureFile = fullfile(featureDirectory, [name_, '.mat']);
                                features2 = zeros(aa * pp, centers);
                                aBinSize = pi / aa;
                                pBinSize = pi / pp;
                                x = floor(orientation(:, :, 1) / aBinSize) + 1;
                                y = floor((orientation(:, :, 2) + pi / 2) / pBinSize) + 1;
                                aBinNo = zeros(1, points);
                                pBinNo = zeros(1, points);
                                for l = 1: 1: points
                                    aBinNo(l) = x(floor(SIFT.l(l, 2)), floor(SIFT.l(l, 1)));
                                    pBinNo(l) = y(floor(SIFT.l(l, 2)), floor(SIFT.l(l, 1)));
                                end
                                binNo = (aBinNo - 1) * pp + pBinNo;
                                count = 0;
                                for s1 = 1: 1: aa
                                    for s2 = 1: 1: pp
                                        count = count + 1;
                                        ind = find(binNo == count);
                                        if (~isempty(ind))
                                            if (LP == -1)
                                                features2(count, :) = max(coef(ind, :), [], 1);
                                            else
                                                features2(count, :) = (sum(coef(ind, :), 1) / length(ind)) .^ (1 / LP);
                                            end
                                        end
                                    end
                                end
                                features2 = features2(:)';
                                features = cat(2, features1, features2);
                                if (normalized)
                                    features = features ./ ((sum(features .^ NP)) ^ (1 / NP)); %#ok<NASGU>
                                end
                                save(featureFile, 'features');
                            end
                        end
                        fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                            j, images, categories);
                    else
                        fprintf('    Image #%03d out of %d images of category %03d existed.\n', ...
                            j, images, categories);
                    end
                end
                totalImages = totalImages + images;
                fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                    categories, categoryName, totalImages);
            end
        end
        fprintf('The pooling step is finished.\n');
        finished = true;
    else
        finished = false;
    end
end

end
