function finished = ExtractLCS(imagePath, descriptorPath, binSize, mag, maxSize)

descriptor = [];
descriptor.categories = 0;
descriptor.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract LCS descriptors.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            descriptor.categories = descriptor.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', descriptor.categories, categoryName);
            ca = descriptor.categories;
            descriptor.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            descriptorDirectory = fullfile(descriptorPath, categoryName);
            if (~isdir(descriptorDirectory))
                mkdir(descriptorDirectory);
            end
            for j = 1: 1: length(imageFiles)
                descriptor.cate(ca).images = descriptor.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                descriptorFile = fullfile(descriptorDirectory, [name_, '.mat']);
                if (~exist(descriptorFile, 'file'))
                    image = imread(imageFile);
                    scale1 = maxSize / size(image, 1);
                    scale2 = maxSize / size(image, 2);
                    if (scale1 < scale2)
                        image = imresize(image, scale1);
                    else
                        image = imresize(image, scale2);
                    end
                    if (size(image, 3) == 1)
                        image = repmat(image, [1, 1, 3]);
                    end
                    image = single(image);
                    [LCS.height, LCS.width, ~] = size(image);
                    [LCS.l, LCS.d] = LocalColorStatistics(image, binSize, mag);
                    save(descriptorFile, 'LCS');
                    fprintf('    Image #%03d out of %d images of category %03d is finished, %05d patches have been extracted.\n', ...
                        j, length(imageFiles), ca, size(LCS.l, 1));
                end
            end
            descriptor.images = descriptor.images + descriptor.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                descriptor.categories, categoryName, descriptor.images);
        end
    end
    fprintf('Extraction of LCS descriptors is finished.\n');
    finished = true;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
