function finished = KGMM(descriptorPath, codebookPath, codebookFileName, codebookFileExpansion, ...
    maxDescriptors, preservedDimensions, kGMM, maxIterations, factor)

[descriptor, going] = LoadDescriptors(descriptorPath, -1, false);

if (going)
    if (~isdir(codebookPath))
        mkdir(codebookPath);
    end
    for p = 1: 1: length(factor)
        centers = kGMM / factor(p);
        codebookName = [codebookFileName, num2str(centers), codebookFileExpansion];
        codebookFile = fullfile(codebookPath, codebookName);
        if (~exist(codebookFile, 'file'))
            fprintf('Begin collecting points for clustering.\n');
            categories = descriptor.categories;
            if (~exist('point', 'var'))
                list = cell(1, categories);
                maxImagesPerCategory = 0;
                for i = 1: 1: categories
                    list{i}.list = randperm(descriptor.cate(i).images);
                    if (maxImagesPerCategory < descriptor.cate(i).images)
                        maxImagesPerCategory = descriptor.cate(i).images;
                    end
                end
                points = 0;
                imagesPerCategory = 0;
                maxDescriptors_ = maxDescriptors * 1048576;
                while (imagesPerCategory < maxImagesPerCategory && (maxDescriptors_ <= 0 || points < maxDescriptors_))
                    imagesPerCategory = imagesPerCategory + 1;
                    for i = 1: 1: categories
                        if (imagesPerCategory <= descriptor.cate(i).images)
                            image = descriptor.cate(i).image(list{i}.list(imagesPerCategory));
                            LCSDescriptor = load(image.path);
                            LCS = LCSDescriptor.LCS;
                            points = points + size(LCS.d, 1);
                        end
                    end
                    fprintf('%d rounds finished, %d points collected.\n', imagesPerCategory, points);
                end
                point = zeros(points, preservedDimensions);
                points = 0;
                for i = 1: 1: categories
                    for j = 1: 1: imagesPerCategory
                        if (j <= descriptor.cate(i).images)
                            image = descriptor.cate(i).image(list{i}.list(j));
                            LCSDescriptor = load(image.path);
                            LCS = LCSDescriptor.LCS;
                            point(points + 1: points + size(LCS.d, 1), :) = LCS.d(:, 1: preservedDimensions);
                            points = points + size(LCS.d, 1);
                        end
                    end
                    fprintf('Category #%d is collected.\n', i);
                end
            end
            fprintf('Clustering begins, there are %d points in total.\n', points);
            tic;
            [GMM.means, GMM.covariances, GMM.priors] = ...
                vl_gmm(point', centers, 'MaxNumIterations', maxIterations);
            toc;
            save(codebookFile, 'GMM');
            fprintf('Codebook with %d entries has been constructed.\n', centers);
        else
            fprintf('Codebook is existed.\n');
        end
    end
    finished = true;
else
    finished = false;
end

end
