function finished = ExtractSIFT(imagePath, descriptorPath, binSize, mag, maxSize)

scale = binSize / mag;

descriptor = [];
descriptor.categories = 0;
descriptor.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract SIFT descriptors.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            descriptor.categories = descriptor.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', descriptor.categories, categoryName);
            ca = descriptor.categories;
            descriptor.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            descriptorDirectory = fullfile(descriptorPath, categoryName);
            if (~isdir(descriptorDirectory))
                mkdir(descriptorDirectory);
            end
            for j = 1: 1: length(imageFiles)
                descriptor.cate(ca).images = descriptor.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                descriptorFile = fullfile(descriptorDirectory, [name_, '.mat']);
                if (~exist(descriptorFile, 'file'))
                    image = imread(imageFile);
                    scale1 = maxSize / size(image, 1);
                    scale2 = maxSize / size(image, 2);
                    if (scale1 < scale2)
                        image = imresize(image, scale1);
                    else
                        image = imresize(image, scale2);
                    end
                    if (size(image, 3) == 3)
                        image = rgb2gray(image);
                    end
                    image = single(image);
                    [SIFT.height, SIFT.width, ~] = size(image);
                    SIFT.parts = 1;
                    [SIFT.l, SIFT.d] = vl_dsift(vl_imsmooth( ...
                        image, sqrt(max(scale .^ 2 - 0.25, 0))), 'size', binSize, 'step', mag, 'floatdescriptors');
                    SIFT.l = SIFT.l';
                    SIFT.d = SIFT.d';
                    for l = 1: 1: size(SIFT.d, 1)
                        ss = sum(SIFT.d(l, :));
                        if (ss > 0)
                            SIFT.d(l, :) = SIFT.d(l, :) / ss;
                        end
                        SIFT.d(l, :) = SIFT.d(l, :) .^ (1 / 2);
                    end
                    SIFT.d = SIFT.d * 512;
                    save(descriptorFile, 'SIFT');
                    fprintf('    Image #%03d out of %d images of category %03d is finished, %05d patches have been extracted.\n', ...
                        j, length(imageFiles), ca, size(SIFT.l, 1));
                end
            end
            descriptor.images = descriptor.images + descriptor.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                descriptor.categories, categoryName, descriptor.images);
        end
    end
    fprintf('Extraction of SIFT descriptors is finished.\n');
    finished = true;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
