function finished = PCA(descriptorPath, pcoefficientPath, pcoefficientFilename, pdescriptorPath, maxDescriptors)

[descriptor, going] = LoadDescriptors(descriptorPath, -1, false);

if (going)
    pcoefficientDirectory = pcoefficientPath;
    if (~isdir(pcoefficientDirectory))
        mkdir(pcoefficientDirectory);
    end
    pcoefficientFile = fullfile(pcoefficientPath, pcoefficientFilename);
    if (~exist(pcoefficientFile, 'file'))
        fprintf('Begin collecting points for PCA.\n');
        categories = descriptor.categories;
        list = cell(1, categories);
        maxImagesPerCategory = 0;
        for i = 1: 1: categories
            list{i}.list = randperm(descriptor.cate(i).images);
            if (maxImagesPerCategory < descriptor.cate(i).images)
                maxImagesPerCategory = descriptor.cate(i).images;
            end
        end
        points = 0;
        imagesPerCategory = 0;
        maxDescriptors_ = maxDescriptors * 1048576;
        while (imagesPerCategory < maxImagesPerCategory && (maxDescriptors_ <= 0 || points < maxDescriptors_))
            imagesPerCategory = imagesPerCategory + 1;
            for i = 1: 1: categories
                if (imagesPerCategory <= descriptor.cate(i).images)
                    image = descriptor.cate(i).image(list{i}.list(imagesPerCategory));
                    SIFTDescriptor = load(image.path);
                    SIFT = SIFTDescriptor.SIFT;
                    points = points + size(SIFT.d, 1);
                end
            end
            fprintf('%d rounds finished, %d points collected.\n', imagesPerCategory, points);
        end
        point = zeros(points, size(SIFT.d, 2));
        points = 0;
        for i = 1: 1: categories
            for j = 1: 1: imagesPerCategory
                if (j <= descriptor.cate(i).images)
                    image = descriptor.cate(i).image(list{i}.list(j));
                    SIFTDescriptor = load(image.path);
                    SIFT = SIFTDescriptor.SIFT;
                    point(points + 1: points + size(SIFT.d, 1), :) = SIFT.d(:, :);
                    points = points + size(SIFT.d, 1);
                end
            end
            fprintf('Category #%d is collected.\n', i);
        end
        fprintf('PCA begins, there are %d points in total.\n', points);
        coefficient = princomp(point);
        save(pcoefficientFile, 'coefficient');
        fprintf('PCA with %d dimensions is finished.\n', size(coefficient, 1));
    else
        coefficient = load(pcoefficientFile);
        coefficient = coefficient.coefficient;
        fprintf('PCA is existed.\n');
    end
    for i = 1: 1: descriptor.categories
        descriptorDirectory = fullfile(descriptorPath, descriptor.cate(i).name);
        pdescriptorDirectory = fullfile(pdescriptorPath, descriptor.cate(i).name);
        if (~isdir(pdescriptorDirectory))
            mkdir(pdescriptorDirectory);
        end
        for j = 1: 1: descriptor.cate(i).images
            descriptorFile = fullfile(descriptorDirectory, [descriptor.cate(i).image(j).name, '.mat']);
            pdescriptorFile = fullfile(pdescriptorDirectory, [descriptor.cate(i).image(j).name, '.mat']);
            if (~exist(pdescriptorFile, 'file'))
                SIFTDescriptor = load(descriptorFile);
                SIFT = SIFTDescriptor.SIFT;
                SIFT.d = SIFT.d * coefficient;
                save(pdescriptorFile, 'SIFT');
                fprintf('    Image #%03d of category %03d is finished.\n', j, i);
            else
                fprintf('    Image #%03d of category %03d existed.\n', j, i);
            end
        end
        fprintf('PCA reduction of category %d is finished.\n', i);
    end
    finished = true;
else
    finished = false;
end

end
