function finished = OrientationVisualization(imagePath, orientationPath, mapPath, maxSize)

if (exist(imagePath, 'dir') && exist(orientationPath, 'dir'))
    fprintf('Begin to plot orientation maps.\n');
    categoryDirectories = dir(imagePath);
    categories = 0;
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            categories = categories + 1;
            fprintf('Category # %03d: %s, begins.\n', categories, categoryName);
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            orientationDirectory = fullfile(orientationPath, categoryName);
            mapDirectory = fullfile(mapPath, categoryName);
            if (~isdir(mapDirectory))
                mkdir(mapDirectory);
            end
            for j = 1: 1: length(imageFiles)
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                orientationFile = fullfile(orientationDirectory, [name_, '.mat']);
                mapFile = fullfile(mapDirectory, [name_, '.png']);
                if (~exist(mapFile, 'file'))
                    image = imread(imageFile);
                    scale1 = maxSize / size(image, 1);
                    scale2 = maxSize / size(image, 2);
                    if (scale1 < scale2)
                        image = imresize(image, scale1);
                    else
                        image = imresize(image, scale2);
                    end
                    orientation = load(orientationFile);
                    orientation = orientation.orientation;
                    width = size(image, 2);
                    height = size(image, 1);
                    %image = rgb2gray(image);
                    imtool(image);
                    info = impixelinfo(imgcf);
                    for x = 1: 1: width
                        for y = 1: 1: height
                            theta = orientation(x, y, 1);
                            phi = orientation(x, y, 2);
                            if (theta ~= 0 || phi ~= 0)
                                x_ = cos(phi) * cos(theta);
                                y_ = sin(phi);
                                z_ = - cos(phi) * sin(theta);
                                length_ = 100;
                                %annotation('arrow', p1, p2, 'LineStyle', '-', 'color', [1 0 0]);
                                annotation(imgcf, 'arrow',[0.2 0.8],[0.2 0.1],'LineStyle','-','color',[1 0 0]);
                            end
                        end
                    end
                    imwrite(mapFile, image, 'png');
                    fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                        j, length(imageFiles), categories);
                end
            end
            fprintf('Category # %03d: %s, is finished.\n', categories, categoryName);
        end
    end
    fprintf('Extraction of orientations is finished.\n');
    finished = true;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
