function ModelTesting(annotationPath, featurePath, modelPath, modelFilename, resultPath, binSize, KNN)

if (~isdir(resultPath))
    mkdir(resultPath);
end

modelFile = fullfile(modelPath, modelFilename);
if (exist(modelFile, 'file'))
    model = load(modelFile);
    model = model.model;
    files = dir(fullfile(annotationPath, '*.mat'));
    acc = zeros(1, length(files));
    for i = 1: 1: length(files)
        [~, name] = fileparts(files(i).name);
        annotationFile = fullfile(annotationPath, [name, '.mat']);
        featureFile = fullfile(featurePath, [name, '.mat']);
        resultFile = fullfile(resultPath, [name, '.mat']);
        if (~exist(resultFile, 'file'))
            annotation = load(annotationFile);
            annotation = annotation.annotation;
            feature = load(featureFile);
            feature = feature.feature;
            features = size(feature.SIFT.d, 2);
            data = zeros(features, model.dataDim);
            label = zeros(features, model.labelDim);
            count = 0;
            for f = 1: 1: features
                x = feature.SIFT.l(1, f);
                y = feature.SIFT.l(2, f);
                reg = annotation.map(y, x);
                if (reg > 0)
                    plane = (sum(annotation.region(reg).vector .^ 2) == 0);
                    if (~plane)
                        complete = true;
                        for a = x - floor(binSize / 2): 1: x + floor(binSize / 2)
                            for b = y - floor(binSize / 2): 1: y + floor(binSize / 2)
                                if (annotation.map(b, a) ~= reg)
                                    complete = false;
                                end
                            end
                        end
                        if (complete && annotation.region(reg).vector(3) <= 0)
                            count = count + 1;
                            data(count, :) = feature.SIFT.d(:, count)';
                            label(count, :) = annotation.region(reg).vector(:);
                        end
                    end
                end
            end
            data = data(1: count, :);
            label = label(1: count, :);
            label_ = zeros(size(label));
            if (KNN > 0)
                dist = 1 - data * model.data';
            else
                coor = zeros(count, 2);
                coor(:, 1) = random('unif', 0, pi, count, 1);
                coor(:, 2) = random('unif', -pi / 2, pi / 2, count, 1);
            end
            for l = 1: 1: count
                if (KNN > 0)
                    d = dist(l, :);
                    [~, ind] = sort(d, 'ascend');
                    coor_ = mean(model.coor(ind(1: KNN), :), 1);
                else
                    coor_ = coor(l, :);
                end
                label_(l, 1) = cos(coor_(2)) * cos(coor_(1));
                label_(l, 2) = sin(coor_(2));
                label_(l, 3) = - cos(coor_(2)) * sin(coor_(1));
            end
            result.label = label;
            result.label_ = label_;
            save(resultFile, 'result');
        else
            result = load(resultFile);
            result = result.result;
            count = size(result.label, 1);
        end
        acc(i) = mean(acos(sum(result.label .* result.label_, 2))) / pi;
        if (count == 0)
            acc(i) = 0.50;
        end
        fprintf('Finished testing %s, accuracy = %0.6f. \n', featureFile, acc(i) * 100);
    end
    fprintf('Mean accuracy = %0.6f. \n', mean(acc) * 100);
else
    ferror('Model file %s does not exist.\n', modelFile);
end

end
