function ModelTraining(annotationPath, featurePath, modelPath, modelFilename, binSize, samples)

if (~isdir(modelPath))
    mkdir(modelPath);
end
modelFile = fullfile(modelPath, modelFilename);
if (~exist(modelFile, 'file'))
    files = dir(fullfile(annotationPath, '*.mat'));
    model.data = [];
    model.label = [];
    model.coor = [];
    for i = 1: 1: length(files)
        [~, name] = fileparts(files(i).name);
        annotationFile = fullfile(annotationPath, [name, '.mat']);
        featureFile = fullfile(featurePath, [name, '.mat']);
        annotation = load(annotationFile);
        annotation = annotation.annotation;
        feature = load(featureFile);
        feature = feature.feature;
        features = size(feature.SIFT.d, 2);
        data_ = zeros(features, size(feature.SIFT.d, 1));
        label_ = zeros(features, 3);
        coor_ = zeros(features, 2);
        count = 0;
        for f = 1: 1: features
            x = feature.SIFT.l(1, f);
            y = feature.SIFT.l(2, f);
            reg = annotation.map(y, x);
            if (reg > 0)
                plane = (sum(annotation.region(reg).vector .^ 2) == 0);
                if (~plane)
                    complete = true;
                    for a = x - floor(binSize / 2): 1: x + floor(binSize / 2)
                        for b = y - floor(binSize / 2): 1: y + floor(binSize / 2)
                            if (annotation.map(b, a) ~= reg)
                                complete = false;
                            end
                        end
                    end
                    if (complete && annotation.region(reg).vector(3) <= 0)
                        count = count + 1;
                        data_(count, :) = feature.SIFT.d(:, count)';
                        label_(count, :) = annotation.region(reg).vector(:);
                        vector = annotation.region(reg).vector;
                        coor_(count, 1) = 0;
                        if (vector(1) > 0 && vector(3) == 0)
                            coor_(count, 1) = 0;
                        elseif (vector(1) < 0 && vector(3) == 0)
                            coor_(count, 1) = pi - 1e-4;
                        elseif (vector(1) == 0)
                            coor_(count, 1) = pi / 2;
                        elseif (vector(1) > 0)
                            coor_(count, 1) = atan(abs(vector(3) / vector(1)));
                        elseif (vector(1) < 0)
                            coor_(count, 1) = pi - atan(abs(vector(3) / vector(1)));
                        end
                        coor_(count, 2) = min(asin(vector(2)), pi / 2 - 1e-4);
                    end
                end
            end
        end
        model.data = cat(1, model.data, data_(1: count, :));
        model.label = cat(1, model.label, label_(1: count, :));
        model.coor = cat(1, model.coor, coor_(1: count, :));
        fprintf('Finished loading %s. \n', featureFile);
    end
    if (size(model.data, 1) < samples)
        samples = size(model.data, 1);
        perm = 1: samples;
    else
        perm = randperm(size(model.data, 1));
    end
    model.data = model.data(perm(1: samples), :);
    model.label = model.label(perm(1: samples), :);
    model.coor = model.coor(perm(1: samples), :);
    model.samples = samples;
    model.dataDim = size(model.data, 2);
    model.labelDim = size(model.label, 2); %#ok<STRNU>
    save(modelFile, 'model');
end

end
